/*
 * Copyright 2008 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that this copyright notice and this permission notice appear in
 * supporting electronic documentation.
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR AUTHORS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <stdlib.h>
#include "dump.h"

void dump_init(struct dump *dump)
{
    dump->pciid = 0;
    dump->cmmregs = 0;
    dump->cixregs = 0;
    INIT_LIST_HEAD(&dump->mmregs);
    INIT_LIST_HEAD(&dump->ixregs);
}

void dump_fini(struct dump *dump)
{
    struct mmreg *mmreg, *nmm;
    struct ixreg *ixreg, *nix;

    list_for_each_entry_safe(mmreg, nmm, &dump->mmregs, list) {
        list_del(&mmreg->list);
        free(mmreg);
    }
    list_for_each_entry_safe(ixreg, nix, &dump->ixregs, list) {
        list_del(&ixreg->list);
        free(ixreg);
    }
    dump_init(dump);
}

void dump_add_mmreg(struct dump *dump, struct mmreg *mmreg)
{
    list_add_tail(&mmreg->list, &dump->mmregs);
}

void dump_add_ixreg(struct dump *dump, struct ixreg *ixreg)
{
    list_add_tail(&ixreg->list, &dump->ixregs);
}

struct mmreg *mmreg_new(unsigned o, unsigned v)
{
    struct mmreg *mmreg;

    mmreg = malloc(sizeof(*mmreg));
    if (mmreg == NULL)
        return NULL;
    mmreg->offset = o;
    mmreg->value = v;
    return mmreg;
}

struct ixreg *ixreg_new(unsigned idx, unsigned ida, unsigned o, unsigned v)
{
    struct ixreg *ixreg;

    ixreg = malloc(sizeof(*ixreg));
    if (ixreg == NULL)
        return NULL;
    ixreg->index = idx;
    ixreg->data = ida;
    ixreg->offset = o;
    ixreg->value = v;
    return ixreg;
}
