/*
 * Copyright 2008 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that this copyright notice and this permission notice appear in
 * supporting electronic documentation.
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR AUTHORS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <pciaccess.h>
#include "radeon.h"
#include "dump.h"

static void usage(void)
{
    printf("Usage: radeondump OPTIONS FILE\n");
    printf("  -d dump, file list the registers that will be dumped\n");
    exit(-1);
}

int main(int argc, char *argv[])
{
    struct radeon radeon;
    struct dump dump;
    char prefix[256];
    char name[256];

    if (argc < 2) {
        usage();
    }
    if (radeon_probe(&radeon)) {
        fprintf(stderr, "Found no graphics card (dumb tool)!\n");
        return 1;
    }
    if (!strcmp(argv[1], "-d")) {
        if (argc != 4) {
            radeon_close(&radeon);
            usage();
        }
        dump_xml_read(&dump, argv[2]);
        if (radeon_dump(&radeon, &dump)) {
            fprintf(stderr, "Error while dumping!\n");
            goto out;
        }
        /* wo would use this prog to do a buffer overflow attack ? :) */
        sprintf(prefix, "%s-0x%04X", argv[3], radeon.dev->device_id);
        /* i don't think anyone is going to make more than 128 dump :) */
        for (int i = 0; i < 128; i++) {
            FILE *file;
            sprintf(name, "%s-%d", prefix, i);
            file = fopen(name, "r");
            if (file == NULL)
                break;
            fclose(file);
        }
        dump_xml_write(&dump, name);
    }
out:
    radeon_close(&radeon);
    return 0;
}

