/*
 * Copyright © 2007 Jerome Glisse <glisse@freedesktop.org>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "radeon.h"

int rv515_mmrd32(struct radeon *radeon, struct mmreg *mmreg)
{
    mmreg->value = radeon_mmio_rd32(radeon, mmreg->offset);
    return 0;
}

int rv515_mmwr32(struct radeon *radeon, struct mmreg *mmreg)
{
    radeon_mmio_wr32(radeon, mmreg->offset, mmreg->value);
    return 0;
}

int rv515_ixrd32(struct radeon *radeon, struct ixreg *ixreg)
{
    switch (ixreg->index) {
    case 0x8:
        /* CLKIND */
        radeon_mmio_wr32(radeon, ixreg->index, ixreg->offset & 0x3F);
        ixreg->value = radeon_mmio_rd32(radeon, 0xC);
        break;
    case 0x30:
        /* PCIEIND */
        radeon_mmio_wr32(radeon, ixreg->index, ixreg->offset & 0x7FF);
        ixreg->value = radeon_mmio_rd32(radeon, 0x34);
        break;
    case 0x70:
        /* MCIND */
        radeon_mmio_wr32(radeon, ixreg->index, ixreg->offset & 0xFFFF);
        ixreg->value = radeon_mmio_rd32(radeon, 0x74);
        break;
    default:
        fprintf(stderr, "(%s:%d) Unknown index register: 0x%08X\n",
                __func__, __LINE__, ixreg->index);
        return -EINVAL;
    }
    return 0;
}

int rv515_ixwr32(struct radeon *radeon, struct ixreg *ixreg)
{
    switch (ixreg->index) {
    case 0x8:
        /* CLKIND */
        radeon_mmio_wr32(radeon, ixreg->index, (ixreg->offset & 0x3F)|(1<<7));
        radeon_mmio_wr32(radeon, 0xC, ixreg->value);
        break;
    case 0x30:
        /* PCIEIND */
        radeon_mmio_wr32(radeon, ixreg->index, (ixreg->offset & 0x7FF));
        radeon_mmio_wr32(radeon, 0x34, ixreg->value);
        break;
    case 0x70:
        /* MCIND */
        radeon_mmio_wr32(radeon, ixreg->index,(ixreg->offset & 0xFFFF)|(1<<23));
        radeon_mmio_wr32(radeon, 0x74, ixreg->value);
        break;
    default:
        fprintf(stderr, "(%s:%d) Unknown index register: 0x%08X\n",
                __func__, __LINE__, ixreg->index);
        return -EINVAL;
    }
    return 0;
}
